<?xml version='1.0' encoding='UTF-8'?>
<!-- Tools for the blog entries -->
<xsl:stylesheet
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	version="1.0"

	xmlns:xuff='http://www.stellated.com/xuff'
	exclude-result-prefixes='xuff'
	>

	<xsl:include href='datetools.xslt'/>

	<xsl:variable
		name='all-links'
		select='document("temp/all-links.xml")'
		/>

	<xsl:variable name='cats' select='document("categories.xml")/categories/category' />

	<!--
	  - Blog entries. Convert .bx to .px
	  -->

	<xsl:template match='entry'>
		<xsl:param name='blogmode'>brief</xsl:param>

		<xsl:variable name='permhref'>
			<xsl:choose>
				<xsl:when test='body/more'>
					<xsl:value-of select='concat("blog/", @when, ".html")' />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select='concat("blog/", substring(@when,1,6), ".html#e", @when)' />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<!-- YUK! We used to use hash-values of just the ISO8601 time, but that's not
		     valid as an id= , so we put an 'e' in front, and this is here so old links
		     will still work. -->
		<xsl:if test='substring(@when,1,8) &lt; 20021206'>
			<a name='{@when}'/>
		</xsl:if>

		<div class='blog-entry' id='e{@when}'>
		<history><what when='{@when}'/></history>

		<!-- Start Head -->
		<div class='head'>
			<!-- Title -->
			<xsl:if test='title'>
				<h3 class='title'>
					<a class='s' href='{$permhref}' rel='bookmark'>
						<xsl:apply-templates select='title/*|title/text()' />
					</a>
				</h3>
			</xsl:if>

			<!-- Date -->
			<p class='date'>
				<xsl:call-template name='long_date_with_day'>
					<xsl:with-param name='date' select='@when' />
				</xsl:call-template>
			</p>
		</div>
		<!-- End Head -->

		<!-- Body -->

		<xsl:if test='$blogmode="brief"'>
			<xsl:apply-templates select='body/*' />
		</xsl:if>

		<xsl:if test='$blogmode="full"'>
			<xsl:apply-templates select='body/*[name()!="more"] | body/more/*' />
		</xsl:if>

		<!-- End Body -->

		<!-- Start Footer -->
		<div class='foot'>
			<span class='via'>

			<!-- Categories -->
			<xsl:if test='category'>
				in:
				<xsl:for-each select='category'>
					<xsl:if test='position() &gt; 1'>, </xsl:if>

					<xsl:variable name='catid' select='text()' />
					<!-- A message if the category doesn't exist. -->
					<xsl:if test='count($cats[@id=$catid])=0'>
						<xsl:message>
							<xsl:text>No such category "</xsl:text>
							<xsl:value-of select='$catid'/>
							<xsl:text>" at </xsl:text>
							<xsl:call-template name='pinpoint'/>
							<xsl:text>, choose from </xsl:text>
							<xsl:for-each select='$cats'>
								<xsl:sort select='@id' />
								<xsl:value-of select='@id' />
								<xsl:text> </xsl:text>
							</xsl:for-each>
						</xsl:message>
					</xsl:if>

					<xsl:variable name='cat' select='$cats[@id=$catid]' />
					<xsl:variable name='tag'>
						<xsl:choose>
							<xsl:when test='$cat/tag'>
								<xsl:value-of select='$cat/tag' />
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select='translate($cat/name, "ABCDEFGHIJKLMNOPQRSTUVWXYZ", "abcdefghijklmnopqrstuvwxyz")' />
							</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>

					<a href='blog/cat_{$catid}.html'>
						<xsl:value-of select='$tag' />
					</a>
				</xsl:for-each>
			</xsl:if>

			<!-- Separator for category and via. -->
			<xsl:if test='category'>
				<xsl:if test='via[@id|@href|text()|*]'>
					&#160;&#160;/&#160;&#160;
				</xsl:if>
			</xsl:if>

			<!-- Via -->
			<xsl:if test='via[@id|@href|text()|*]'>
				<xsl:call-template name='do_via'/>
			</xsl:if>
			</span>

			<!-- Reactions -->
			<php>
				$eid = 'e<xsl:value-of select="@when"/>';
			</php>
				<span class='react'>
					<gtgt/><space/>
					<a
						href='reactor/comment.php?entryid=e{@when}'
						onClick="openComments('e{@when}'); return false;"
						onMouseOver="window.status='react to this entry'; return true"
						onMouseOut="window.status=' '; return true"
						>
						<php ifnot='react'>echo($ror->CommentLabel($eid));</php>
					</a>
				</span>
			<php>
				if ($ror->AreCommentsInline($eid)) {
					$ror->HandlePostedComment();
					echo $ror->RenderComments($eid, 'inc/inlinecomment.html');
				}
			</php>

		</div>
		<!-- End Footer -->

		</div>
	</xsl:template>

	<!-- A <more> section becomes a link with a count of other content. -->

	<xsl:template match='more'>
		<xsl:variable name='census' select='count(p|ul/li|ol/li|code|quote)' />

		<p class='more'>
			<a class='s' href='blog/{ancestor::entry/@when}.html'>
				<gtgt/><xsl:text> more... (</xsl:text>

				<xsl:choose>
					<xsl:when test='$census = 0'>
						<xsl:text>empty!</xsl:text>
					</xsl:when>
					<xsl:when test='$census = 1'>
						<xsl:text>1 paragraph</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select='$census' />
						<xsl:text> paragraphs</xsl:text>
					</xsl:otherwise>
				</xsl:choose>
				<xsl:text>)</xsl:text>
			</a>
		</p>
	</xsl:template>

	<xsl:template name='do_via'>
		<xsl:text>via: </xsl:text>
		<xsl:for-each select='via'>
			<xsl:if test='position() &gt; 1'>, and </xsl:if>
			<xsl:choose>
			<xsl:when test='@id'>
				<!-- Print a message if the id attribute is bogus. -->
				<xsl:if test='count($all-links//link[@id=current()/@id])=0'>
					<xsl:message>
						<xsl:text>*** Unknown via id: </xsl:text>
						<xsl:value-of select='@id'/>
						<xsl:text> ***</xsl:text>
					</xsl:message>
				</xsl:if>
				<xsl:for-each select='$all-links//link[@id=current()/@id]'>
					<a href='{@href}'><xsl:copy-of select='title/*|title/text()'/></a>
				</xsl:for-each>
			</xsl:when>
			<xsl:when test='@href'>
				<a href='{@href}'><xsl:copy-of select='*[name()!="via"]|text()'/></a>
			</xsl:when>
			<xsl:when test='text()|*[name()!="via"]'>
				<span><xsl:copy-of select='text()|*[name()!="via"]'/></span>
			</xsl:when>
			</xsl:choose>

			<xsl:if test='via'>
				<xsl:call-template name='do_via'/>
			</xsl:if>

		</xsl:for-each>
	</xsl:template>

	<xsl:template match='quick'>
		<p class='quick'>
			<pilcrow/><nbsp/><nbsp/>
			<xsl:apply-templates select='text()|*[name()!="via"]'/>
			<xsl:if test='via[@id|@href|text()|*]'>
				<nbsp/><span class='via'><i>
				<xsl:text>(</xsl:text>
				<xsl:call-template name='do_via'/>
				<xsl:text>)</xsl:text>
				</i></span>
			</xsl:if>
		</p>
	</xsl:template>

</xsl:stylesheet>
