<?xml version='1.0' encoding='UTF-8'?>
<!-- Convert a bunch of blog/entry's from a .bx file to a .px file. -->
<!-- This produces the yearly blog archive page. -->
<xsl:stylesheet
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	version="1.0"

	xmlns:xuff='http://www.stellated.com/xuff'
	exclude-result-prefixes='xuff'
	>

	<xsl:include href='blog.xslt'/>
	<xsl:include href='tools.xslt'/>

	<xsl:output method='xml' indent='yes' />

	<xsl:key name='blog-year' match='blog/entry[string(@draft)=""]' use='substring(@when,1,4)' />
	<xsl:key name='blog-month' match='blog/entry[string(@draft)=""]' use='substring(@when,1,6)' />
	<xsl:key name='blog-day' match='blog/entry[string(@draft)=""]' use='substring(@when,1,8)' />

	<xsl:template match='/'>

		<directory name='blog'>
			<!-- For each first entry of the year. -->
			<xsl:for-each select='//blog/entry[count(. | key("blog-year", substring(@when,1,4))[1]) = 1]' >
				<file name='archive{substring(@when,1,4)}.px'>


					<!-- The page structure. -->
					<page title='Blog Archive: {substring(@when,1,4)}' order='{9999-number(substring(@when,1,4))}'>
						<meta name='ROBOTS' content='NOINDEX' />
						<pagemenu file='blogroll.hx' phpinc='blogroll.inc'/>

						<!-- Write a fake history tag to get the copyright correct. -->
						<history><what when='{@when}'/></history>

						<p class='first'>
							These are the
							<xsl:value-of select='count(key("blog-year", substring(@when,1,4)))'/>
							<xsl:text> </xsl:text>
							<a href='blog'>blog</a> entries for
							<xsl:value-of select='substring(@when,1,4)'/>,
							organized by date, newest first, and separated into months.
							There is also a <a href='blog/archiveall.html'>complete archive</a>.
						</p>

						<div class='archive'>
							<!-- For each entry in this year that is the first in the month. -->
							<xsl:for-each select='key("blog-year", substring(@when,1,4))[count(. | key("blog-month", substring(@when,1,6))[1]) = 1]' >
								<xsl:sort select='@when' order='descending'/>

								<xsl:call-template name='month-header' />

								<!-- For each entry in this month that is the first in the day. -->
								<xsl:for-each select='key("blog-month", substring(@when,1,6))[count(. | key("blog-day", substring(@when,1,8))[1]) = 1]' >
									<xsl:sort select='@when' order='descending'/>

									<p class='date'>
										<xsl:call-template name='dayname_in_month'>
											<xsl:with-param name='date'>
												<xsl:value-of select='substring(@when,1,8)'/>
											</xsl:with-param>
										</xsl:call-template>

										<xsl:text>: </xsl:text>
									</p>

									<!-- For each entry in this day. -->
									<xsl:for-each select='key("blog-day", substring(@when,1,8))'>
										<xsl:sort select='@when' order='descending'/>

										<p>
											<a class='s' href='blog/{substring(@when,1,6)}.html#e{@when}'>
												<xsl:copy-of select='title/text()'/>
											</a>
										</p>

									</xsl:for-each>

								</xsl:for-each>


							</xsl:for-each>
						</div>
					</page>
				</file>
			</xsl:for-each>
		</directory>

	</xsl:template>

	<xsl:template name='month-header'>
		<h1>
			<a class='s' href='blog/{substring(@when,1,6)}.html'>
				<xsl:call-template name='long_date_month_year'>
					<xsl:with-param name='date'>
						<xsl:value-of select='substring(@when,1,8)'/>
					</xsl:with-param>
				</xsl:call-template>
				<!--
				<xsl:text> (</xsl:text>
				<xsl:value-of select='count(key("blog-month", substring(@when,1,6)))'/>
				<xsl:text> entries)</xsl:text>
				-->
			</a>
		</h1>
	</xsl:template>

</xsl:stylesheet>
