<?xml version='1.0' encoding='UTF-8'?>
<!-- Convert a bunch of blog/entry's from a .bx file to a .px file. -->
<!-- This produces the category pages. -->
<xsl:stylesheet
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	version="1.0"

	xmlns:xuff='http://www.stellated.com/xuff'
	exclude-result-prefixes='xuff'
	>

	<xsl:include href='blog.xslt'/>
	<xsl:include href='tools.xslt'/>

	<xsl:output method='xml' indent='yes' />

	<xsl:key name='blog-category' match='blog/entry[string(@draft)=""]' use='category' />

	<xsl:template match='/'>
		<xsl:variable name='blog' select='.'/>

		<directory name='blog'>
			<!-- For every defined category, -->
			<xsl:for-each select='document("categories.xml")//category' >
				<xsl:variable name='catid' select='@id' />
				<xsl:variable name='catname' select='name' />

				<!-- make a file for that category. -->
				<file name='cat_{$catid}.px'>

					<!-- The page structure. -->
					<page title='Category: {$catname}' sitemap='no'>
						<pagemenu file='blogroll.hx' phpinc='blogroll.inc' />

						<p>These are my blog postings about <xsl:value-of select='about'/>:</p>

						<div class='category'>

							<!-- This for-each nonsense is needed to get the context node right for the key function! -->
							<xsl:for-each select='$blog'>
								<xsl:for-each select='key("blog-category", $catid)'>
									<xsl:sort select='@when' order='descending'/>

									<p class='date'>
										<xsl:call-template name='med_date'>
											<xsl:with-param name='date'>
												<xsl:value-of select='substring(@when,1,8)'/>
											</xsl:with-param>
										</xsl:call-template>

										<xsl:text>: </xsl:text>
									</p>

									<p>
										<a class='s' href='blog/{substring(@when,1,6)}.html#e{@when}'>
											<xsl:copy-of select='title/text()'/>
										</a>
									</p>

								</xsl:for-each>
							</xsl:for-each>

						</div>

						<xsl:if test='related'>
							<p>You might also like these related categories:
								<xsl:for-each select='related'>
									<xsl:variable name='othercat' select='document("categories.xml")//category[@id=current()]' />
									<xsl:if test='position()>1'>, </xsl:if>
									<a href='blog/cat_{$othercat/@id}.html'><xsl:value-of select='$othercat/name'/></a>
								</xsl:for-each>
								<xsl:text>.</xsl:text>
							</p>
						</xsl:if>

						<p>
							Didn't find what you were looking for?
							<a href='site/search.html'>Search this site</a> with Google.
						</p>

						<googleads channel='category' />
					</page>
				</file>
			</xsl:for-each>

			<!-- A page for uncategorized entries. -->
			<file name='cat_none.px'>

				<!-- The page structure. -->
				<page title='Uncategorized postings'>
					<pagemenu file='blogroll.hx' phpinc='blogroll.inc' />

					<p>These are my uncategorized blog postings:</p>

					<div class='category'>

						<!-- This for-each nonsense is needed to get the context node right for the key function! -->
						<xsl:for-each select='$blog//entry[string(@draft)=""][count(category)=0]'>
							<xsl:sort select='@when' order='descending'/>

							<p class='date'>
								<xsl:call-template name='med_date'>
									<xsl:with-param name='date'>
										<xsl:value-of select='substring(@when,1,8)'/>
									</xsl:with-param>
								</xsl:call-template>

								<xsl:text>: </xsl:text>
							</p>

							<p>
								<a class='s' href='blog/{substring(@when,1,6)}.html#e{@when}'>
									<xsl:copy-of select='title/text()'/>
								</a>
							</p>

						</xsl:for-each>

					</div>

					<p>
						What better way to conquer the chaos of uncategorized postings than by
						<a href='site/search.html'>searching this site</a> with Google?
					</p>

					<googleads channel='category' />
				</page>
			</file>

			<!-- A page summarizing all the categories. -->
			<file name='categories.px'>
				<page title='Blog Categories'>
					<pagemenu file='blogroll.hx' phpinc='blogroll.inc' />

					<p>Here is my attempt at categorizing and making sense of my blog postings:</p>

					<div class='catlist'>
						<!-- A line for each category. -->
						<xsl:for-each select='document("categories.xml")//category' >
							<xsl:sort select='name' />
							<xsl:variable name='catid' select='@id' />
							<xsl:variable name='catname' select='name' />
							<xsl:variable name='catnum' select='count($blog//entry[category=$catid])' />

							<p class='catname'>
								<a class='s' href='blog/cat_{$catid}.html'>
									<xsl:value-of select='$catname' />
									<span class='dn'>: </span>
								</a>
							</p>
							<p>
								<a class='s' href='blog/cat_{$catid}.html'>
									<xsl:value-of select='$catnum' />
									post<xsl:if test='$catnum != 1'>s</xsl:if> about
									<xsl:value-of select='about' />.
								</a>
							</p>
						</xsl:for-each>

						<!-- And a line for the uncategorized postings. -->
						<xsl:variable name='nones' select='count($blog//entry[count(category)=0])' />
						<xsl:if test='$nones != 0'>
							<p class='catname'>
								<a class='s' href='blog/cat_none.html'>(None)</a>
							</p>
							<p>
								<a class='s' href='blog/cat_none.html'>
									<xsl:value-of select='$nones' />
									post<xsl:if test='$nones != 1'>s</xsl:if>
									that haven't been categorized at all.
								</a>
							</p>
						</xsl:if>
					</div>

					<p>If categories don't do it for you, try
						<a href='site/search.html'>searching this site</a> with Google.
					</p>
				</page>
			</file>

		</directory>

	</xsl:template>

</xsl:stylesheet>
