<?xml version='1.0' encoding='UTF-8'?>
<!-- Convert a bunch of blog/entry's from a .bx file to a .px file. -->
<!-- This produces the main blog page. -->
<xsl:stylesheet
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	version="1.0"

	xmlns:xuff='http://www.stellated.com/xuff'
	exclude-result-prefixes='xuff'
	>

	<xsl:param name='base' />

	<xsl:include href='blog.xslt' />
	<xsl:include href='tools.xslt' />

	<xsl:template match='/'>

		<!-- The page structure. -->
		<page title='Blog'>
			<section title='Blog' order='1'/>
			<meta name='ROBOTS' content='NOINDEX' />

			<!-- geourl.org tags -->
			<meta name="ICBM" content="42.32921,-71.11914" />
			<meta name="geo.position" content="42.32921,-71.11914" />
			<meta name="geo.region" content="US-MA" />
			<meta name="geo.placename" content="Brookline, Massachusetts" />

			<!-- The RSS feeds for this page. -->
			<pagelink
				rel='alternate'
				type='application/rss+xml'
				href='{$base}blog/rss.xml'
				title='RSS feed for Ned&apos;s blog (excerpts)' />
			<pagelink
				rel='alternate'
				type='application/rss+xml'
				href='{$base}blog/rssfull.xml'
				title='RSS feed for Ned&apos;s blog (full text)' />
			<pagelink
				rel='alternate'
				type='application/atom+xml'
				href='blog/atom.xml'
				title='Atom feed for Ned&apos;s blog (full text)' />

			<pagelink rel='icon' href='/favicon.ico' type='image/x-icon' />

			<pagemenu file='blogroll.hx' phpinc='blogroll.inc'/>

			<php>
				require_once('inc/reactor_class.php');
				$ror = new Reactor();
				$ror->PrepareManyComments();
			</php>

			<xsl:for-each select='//blog/entry[string(@draft)=""]'>
				<!-- Do all the blog entries, in descending order. -->
				<xsl:sort select='@when' order='descending'/>

				<xsl:choose>

				<!-- The first 12 entries get listed explicitly. -->
				<xsl:when test='position() &lt; 13'>
					<xsl:apply-templates select='.' />
				</xsl:when>

				<!-- The 13th through 20th entries are links to another page. -->
				<xsl:when test='position() &lt; 21'>
					<xsl:if test='position() = 13'>
						<p><i>Older:</i></p>
					</xsl:if>

					<div class='archive'>
						<p class='date'>
							<xsl:call-template name='dayname_in_month'>
								<xsl:with-param name='date' select='@when' />
							</xsl:call-template>
							<xsl:text>: </xsl:text>
						</p>

						<p>
							<a href='blog/{substring(@when,1,6)}.html#e{@when}' class='s'>
								<xsl:apply-templates select='title/*|title/text()' />
							</a>
						</p>
					</div>
				</xsl:when>

				<!-- The 21st entry is a link to another page. -->
				<xsl:when test='position() = 21'>
					<p><i><a href='blog/{substring(@when,1,6)}.html#e{@when}'>Even older...</a></i></p>
				</xsl:when>

				</xsl:choose>
			</xsl:for-each>

			<xsl:for-each select='//blog/entry[@draft="y"]'>
				<xsl:message>Draft entry: <xsl:value-of select='title/text()'/></xsl:message>
			</xsl:for-each>

			<!-- Technorati proof of claim -->
			<a style="display:none" href="http://www.technorati.com/profile/nedbatch/35006/701ce75e8a768c33e919b777f527e721">Technorati profile</a>

		</page>

	</xsl:template>

</xsl:stylesheet>
