<?xml version='1.0' encoding='UTF-8'?>
<!--
	Convert blog entry's from the blogfeed.xml file to a rss.xml file.
  RSS 1.0 spec: http://web.resource.org/rss/1.0/spec
-->

<xsl:stylesheet
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"

	xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
	xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns="http://purl.org/rss/1.0/"

	version="1.0"

	xmlns:xuff='http://www.stellated.com/xuff'
	exclude-result-prefixes='xuff'
	>

	<xsl:output method='xml' indent='no' />

	<xsl:param name='base' />
	<xsl:param name='sitename' />

	<!-- Set this param to 'y' to get excerpts, else full entries -->
	<xsl:param name='brief' />

	<xsl:include href='blog.xslt'/>
	<xsl:include href='tools.xslt'/>

	<xsl:template match='/feedentries'>

		<xsl:variable name='stylesheet'>
			<xsl:value-of select='$base'/>
			<xsl:choose>
				<xsl:when test='$brief = "y"'>rss2html.xslt</xsl:when>
				<xsl:otherwise>rssfull2html.xslt</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<!-- Link to the stylesheet for people who wander in. -->
		<xsl:processing-instruction name='xml-stylesheet'>
			type="text/xsl"
			href="<xsl:value-of select='$stylesheet'/>"
			media="screen"
		</xsl:processing-instruction>

		<rdf:RDF>
			<channel rdf:about='{$base}/blog'>
				<title>Ned Batchelder's blog</title>
				<link><xsl:value-of select='$base'/>blog</link>
				<description>Ned Batchelder's personal blog.</description>
				<dc:language>en-US</dc:language>
				<image rdf:resource='http://nedbatchelder.com/pix/rss-banner.gif' />

				<items>
					<rdf:Seq>
						<xsl:for-each select='entry'>
							<rdf:li resource='{@what}' />
						</xsl:for-each>
					</rdf:Seq>
				</items>

			</channel>

			<image rdf:about='http://nedbatchelder.com/pix/rss-banner.gif'>
				<title>Ned Batchelder's blog</title>
				<link><xsl:value-of select='$base'/>blog</link>
				<url>http://nedbatchelder.com/pix/rss-banner.gif</url>
			</image>

			<xsl:for-each select='entry'>
				<item rdf:about='{@what}'>
					<title><xsl:value-of select='title'/></title>
					<link><xsl:value-of select='@what'/></link>
					<dc:date>
						<xsl:value-of select='xuff:w3cdtf(string(@when))' />
					</dc:date>
					<description>
						<xsl:choose>
							<xsl:when test='$brief = "y"'>
								<!-- This gets just the text of the paragraph, with no links, style, etc. -->
								<xsl:value-of select='body/*[1]'/>
								<xsl:if test='body/*[2]'>
									<xsl:text> (more..)</xsl:text>
								</xsl:if>
							</xsl:when>
							<xsl:otherwise>
								<xsl:text disable-output-escaping='yes'>&lt;![CDATA[</xsl:text>
								<xsl:apply-templates select='body/*[name()!="more"] | body/more/*'/>
								<xsl:text disable-output-escaping='yes'>]]&gt;</xsl:text>
							</xsl:otherwise>
						</xsl:choose>
					</description>
				</item>
			</xsl:for-each>

		</rdf:RDF>

	</xsl:template>

</xsl:stylesheet>
