<?xml version='1.0' encoding='UTF-8'?>
<!-- Make the per-month blog pages -->
<xsl:stylesheet
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	version="1.0"

	xmlns:xuff='http://www.stellated.com/xuff'
	exclude-result-prefixes='xuff'
	>

	<xsl:include href='datetools.xslt' />
	<xsl:include href='tools.xslt' />
	<xsl:output method='xml' indent='yes' />

	<xsl:key name='blog-month' match='blog/entry[string(@draft)=""]' use='substring(@when,1,6)' />

	<xsl:template match='/'>
		<directory name='blog'>
			<xsl:for-each select='//blog/entry[count(. | key("blog-month", substring(@when,1,6))[1]) = 1]' >

				<xsl:variable name='prev'>
					<xsl:call-template name='prev_month'>
						<xsl:with-param name='date'>
							<xsl:value-of select='substring(@when,1,6)' />
						</xsl:with-param>
					</xsl:call-template>
				</xsl:variable>

				<xsl:variable name='next'>
					<xsl:call-template name='next_month'>
						<xsl:with-param name='date'>
							<xsl:value-of select='substring(@when,1,6)' />
						</xsl:with-param>
					</xsl:call-template>
				</xsl:variable>

				<xsl:variable name='title'>
					<xsl:call-template name='long_date_month_year'>
						<xsl:with-param name='date'>
							<xsl:value-of select='substring(@when,1,8)'/>
						</xsl:with-param>
					</xsl:call-template>
				</xsl:variable>

				<file name='{substring(@when,1,6)}.bx'>
					<blog order='{1000-number(substring(@when,4,3))}' title='{$title}'>

						<!-- If there is a previous month, put in the prev link. -->
						<xsl:if test='key("blog-month", $prev)'>
							<navigation rel='prev' href='blog/{$prev}.html'>
								<xsl:call-template name='long_date_month_year'>
									<xsl:with-param name='date'>
										<xsl:value-of select='$prev' />
									</xsl:with-param>
								</xsl:call-template>
							</navigation>
						</xsl:if>

						<!-- If there is a next month, put in the next link. -->
						<xsl:if test='key("blog-month", $next)'>
							<navigation rel='next' href='blog/{$next}.html'>
								<xsl:call-template name='long_date_month_year'>
									<xsl:with-param name='date'>
										<xsl:value-of select='$next' />
									</xsl:with-param>
								</xsl:call-template>
							</navigation>
						</xsl:if>

						<!-- A summary paragraph. -->
<!--						<p>
							These are all
							<xsl:value-of select='count(key("blog-month", substring(@when,1,6)))'/>
							<xsl:text> </xsl:text>
							<a href='blog'>blog</a> entries for
							<xsl:value-of select='$title'/>,
							organized reverse chronologically.
						</p>
-->
						<!-- Put in all the entries. -->
						<xsl:for-each select='key("blog-month", substring(@when,1,6))'>
							<xsl:copy-of select='.'/>
						</xsl:for-each>

					</blog>
				</file>
			</xsl:for-each>
		</directory>
	</xsl:template>

</xsl:stylesheet>
