<?xml version='1.0' encoding='UTF-8'?>
<!-- Date manipulation tools. -->
<xsl:stylesheet
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:date="http://exslt.org/dates-and-times"
	version="1.0"

	xmlns:xuff='http://www.stellated.com/xuff'
	exclude-result-prefixes='xuff'

	xmlns:local='http://www.stellated.com/local'
	>

	<xsl:include href='day-in-week.xslt' />

	<!--
	  -		8601 to "dd/mm/yyyy"
	  -->

	<xsl:template name="standard_date">
		<xsl:param name="date" />
		<!-- Day -->
		<xsl:value-of select="substring($date, 7, 2)" />
		<xsl:text>/</xsl:text>
		<!-- Month -->
		<xsl:value-of select="substring($date, 5, 2)" />
		<xsl:text>/</xsl:text>
		<!-- Year -->
		<xsl:value-of select="substring($date, 3, 2)" />
	</xsl:template>

	<xsl:template name="short_date">
		<xsl:param name="date" />
		<!-- Month -->
		<xsl:value-of select="substring($date, 5, 2)" />
		<xsl:text>/</xsl:text>
		<!-- Year -->
		<xsl:value-of select="substring($date, 3, 2)" />
	</xsl:template>

	<!--
	  -		8601 to eg, "January 2003"
	  -->

	<xsl:template name="long_date_month_year">
		<xsl:param name="date" />

		<!-- Month -->
		<xsl:variable name="month" select="number(substring($date, 5, 2))" />
		<xsl:choose>
			<xsl:when test="$month=1">January</xsl:when>
			<xsl:when test="$month=2">February</xsl:when>
			<xsl:when test="$month=3">March</xsl:when>
			<xsl:when test="$month=4">April</xsl:when>
			<xsl:when test="$month=5">May</xsl:when>
			<xsl:when test="$month=6">June</xsl:when>
			<xsl:when test="$month=7">July</xsl:when>
			<xsl:when test="$month=8">August</xsl:when>
			<xsl:when test="$month=9">September</xsl:when>
			<xsl:when test="$month=10">October</xsl:when>
			<xsl:when test="$month=11">November</xsl:when>
			<xsl:when test="$month=12">December</xsl:when>
			<xsl:otherwise>INVALID MONTH</xsl:otherwise>
		</xsl:choose>
		<xsl:text> </xsl:text>
		<!-- Year -->
		<xsl:value-of select="substring($date, 1, 4)" />
	</xsl:template>

	<!--
	  -		8601 to eg, "Jan 2003"
	  -->

	<xsl:template name="med_date_month_year">
		<xsl:param name="date" />

		<!-- Month -->
		<xsl:variable name="month" select="number(substring($date, 5, 2))" />
		<xsl:choose>
			<xsl:when test="$month=1">Jan</xsl:when>
			<xsl:when test="$month=2">Feb</xsl:when>
			<xsl:when test="$month=3">Mar</xsl:when>
			<xsl:when test="$month=4">Apr</xsl:when>
			<xsl:when test="$month=5">May</xsl:when>
			<xsl:when test="$month=6">Jun</xsl:when>
			<xsl:when test="$month=7">Jul</xsl:when>
			<xsl:when test="$month=8">Aug</xsl:when>
			<xsl:when test="$month=9">Sep</xsl:when>
			<xsl:when test="$month=10">Oct</xsl:when>
			<xsl:when test="$month=11">Nov</xsl:when>
			<xsl:when test="$month=12">Dec</xsl:when>
			<xsl:otherwise>INVALID MONTH</xsl:otherwise>
		</xsl:choose>
		<xsl:text> </xsl:text>
		<!-- Year -->
		<xsl:value-of select="substring($date, 1, 4)" />
	</xsl:template>

	<!--
	  -		8601 to eg, "23 January 2003"
	  -->

	<xsl:template name='long_date'>
		<xsl:param name='date' />

		<!-- Day -->
		<xsl:value-of select='number(substring($date, 7, 2))' />
		<xsl:text> </xsl:text>

		<xsl:call-template name='long_date_month_year'>
			<xsl:with-param name='date'>
				<xsl:value-of select='$date'/>
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<!--
	  -		8601 to eg, "23 Jan 2003"
	  -->

	<xsl:template name='med_date'>
		<xsl:param name='date' />

		<!-- Day -->
		<xsl:value-of select='number(substring($date, 7, 2))' />
		<xsl:text> </xsl:text>

		<xsl:call-template name='med_date_month_year'>
			<xsl:with-param name='date'>
				<xsl:value-of select='$date'/>
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<!--
	  -		8601 to eg, "Sunday 23 January 2003"
	  -->

	<xsl:template name='long_date_with_day'>
		<xsl:param name='date' />

		<xsl:variable name='weekday'>
			<xsl:call-template name='date:day-in-week'>
				<xsl:with-param name='date-time'>
					<xsl:value-of select='substring($date, 1, 4)'/>
					<xsl:text>-</xsl:text>
					<xsl:value-of select='substring($date, 5, 2)'/>
					<xsl:text>-</xsl:text>
					<xsl:value-of select='substring($date, 7, 2)'/>
				</xsl:with-param>
			</xsl:call-template>
		</xsl:variable>

		<xsl:choose>
			<xsl:when test='$weekday=1'>Sunday</xsl:when>
			<xsl:when test='$weekday=2'>Monday</xsl:when>
			<xsl:when test='$weekday=3'>Tuesday</xsl:when>
			<xsl:when test='$weekday=4'>Wednesday</xsl:when>
			<xsl:when test='$weekday=5'>Thursday</xsl:when>
			<xsl:when test='$weekday=6'>Friday</xsl:when>
			<xsl:when test='$weekday=7'>Saturday</xsl:when>
			<xsl:otherwise>??<xsl:value-of select='$weekday'/>??</xsl:otherwise>
		</xsl:choose>
		<xsl:text> </xsl:text>

		<xsl:call-template name='long_date'>
			<xsl:with-param name='date' select='$date' />
		</xsl:call-template>

	</xsl:template>

	<!--
	  -		8601 to eg, "Sun 23"
	  -->

	<xsl:template name='dayname_in_month'>
		<xsl:param name='date' />

		<xsl:variable name='weekday'>
			<xsl:call-template name='date:day-in-week'>
				<xsl:with-param name='date-time'>
					<xsl:value-of select='substring($date, 1, 4)'/>
					<xsl:text>-</xsl:text>
					<xsl:value-of select='substring($date, 5, 2)'/>
					<xsl:text>-</xsl:text>
					<xsl:value-of select='substring($date, 7, 2)'/>
				</xsl:with-param>
			</xsl:call-template>
		</xsl:variable>

		<xsl:choose>
			<xsl:when test='$weekday=1'>Sun</xsl:when>
			<xsl:when test='$weekday=2'>Mon</xsl:when>
			<xsl:when test='$weekday=3'>Tue</xsl:when>
			<xsl:when test='$weekday=4'>Wed</xsl:when>
			<xsl:when test='$weekday=5'>Thu</xsl:when>
			<xsl:when test='$weekday=6'>Fri</xsl:when>
			<xsl:when test='$weekday=7'>Sat</xsl:when>
			<xsl:otherwise>??<xsl:value-of select='$weekday'/>??</xsl:otherwise>
		</xsl:choose>

		<xsl:text> </xsl:text>

		<xsl:value-of select='number(substring($date, 7, 2))'/>

	</xsl:template>

	<!--
	  -		The next month: for example, "20030123" -> "200302"
	  -->

	<xsl:template name='next_month'>
		<xsl:param name='date' />

		<xsl:variable name='year' select='substring($date, 1, 4)' />
		<xsl:variable name='month' select='substring($date, 5, 2)' />
		<xsl:variable name='nextmonth0' select='string(number($month)+1)' />
		<xsl:variable name='nextyear'>
			<xsl:choose>
				<xsl:when test='$nextmonth0 = "13"'>
					<xsl:value-of select='string(number($year)+1)' />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select='$year' />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name='nextmonth'>
			<xsl:choose>
				<xsl:when test='$nextmonth0 = "13"'>01</xsl:when>
				<xsl:when test='string-length($nextmonth0) = 1'>0<xsl:value-of select='$nextmonth0'/></xsl:when>
				<xsl:otherwise><xsl:value-of select='$nextmonth0'/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:value-of select='concat($nextyear, $nextmonth)'/>
	</xsl:template>

	<!--
	  -		The previous month: for example, "20030123" -> "200212"
	  -->

	<xsl:template name='prev_month'>
		<xsl:param name='date' />

		<xsl:variable name='year' select='substring($date, 1, 4)' />
		<xsl:variable name='month' select='substring($date, 5, 2)' />
		<xsl:variable name='prevmonth0' select='string(number($month)-1)' />
		<xsl:variable name='prevyear'>
			<xsl:choose>
				<xsl:when test='$prevmonth0 = "0"'>
					<xsl:value-of select='string(number($year)-1)' />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select='$year' />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name='prevmonth'>
			<xsl:choose>
				<xsl:when test='$prevmonth0 = "0"'>12</xsl:when>
				<xsl:when test='string-length($prevmonth0) = 1'>0<xsl:value-of select='$prevmonth0'/></xsl:when>
				<xsl:otherwise><xsl:value-of select='$prevmonth0'/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:value-of select='concat($prevyear, $prevmonth)'/>
	</xsl:template>

</xsl:stylesheet>
