<?xml version='1.0' encoding='UTF-8'?>
<!-- Style a sub page -->
<xsl:stylesheet
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	version="1.0"

	xmlns:xuff='http://www.stellated.com/xuff'
	exclude-result-prefixes='xuff'
	>

<xsl:param name='path' />
<xsl:param name='dpath' />
<xsl:param name='base' />
<xsl:param name='sitename' />
<xsl:param name='redirect' />
<xsl:param name='nophp' />

<xsl:include href='tools.xslt' />
<xsl:include href='outputtools.xslt' />
<xsl:include href='datetools.xslt'/>

<xsl:variable name='pagepaths' select='document("temp/pagepaths.xml")/pages' />
<xsl:variable name='pagepath' select='$pagepaths/page[@path=$path]' />

<xsl:variable name='pagemenus' select='document("temp/pagemenus.xml")/pages' />
<xsl:variable name='pagemenu' select='$pagemenus/page[@path=$path]' />
<xsl:variable name='rootmenu' select='$pagemenus/page[@path="index.px"]' />

<xsl:template match='page'>

	<!-- HEAD -->
	<head>

		<xsl:copy-of select='meta'/>
		<xsl:if test='$redirect'>
			<meta http-equiv='refresh' content='15;url={$redirect}{$dpath}'/>
		</xsl:if>
		<xsl:if test='$base'>
			<base href='{$base}'/>
		</xsl:if>

		<title><xsl:value-of select='concat($sitename, ": ", @title)'/></title>
		<link rel='StyleSheet' href='style.css' type='text/css'/>

		<xsl:for-each select='pagelink'>
			<link>
				<xsl:copy-of select='@*' />
			</link>
		</xsl:for-each>

		<script type='text/javascript' src='main.js'/>
		<script type='text/javascript' src='hilightsearch.js'/>
		<script type='text/javascript' src='hilightfrag.js'/>
	</head>

	<!-- BODY -->

	<body bgcolor='white'>

		<xsl:if test='$redirect'>
			<!-- Some callers want to always put a redirect on the page. -->
			<table
				border='1' style='border: red 2px solid; background: pink'
				width='100%' cellspacing='0' cellpadding='5'>
				<tr><td align='center'>
					<b>This page has moved.  Update your links to:
					<a href='{$redirect}{$dpath}'><xsl:value-of select='$redirect'/><xsl:value-of select='$dpath'/></a></b>
				</td></tr>
			</table>
			<p><img src='1.gif' alt='' width='1' height='300'/></p>
		</xsl:if>

		<table style='table-layout: fixed' width='100%' cellpadding='0' cellspacing='20'>
		<tr>
			<td id='logo' width='120px' align='center'>
				<img src='dodeca3_100.gif' width='100' height='120' alt='[*]' />
			</td>
			<td id='banner'>
				<!-- Above the title: a horizontal version of the root pagemenu. -->
				<span class='path'>
					<a href='index.html'><xsl:value-of select='$sitename'/></a>
					<xsl:if test='$rootmenu/a'>
						<xsl:text> : </xsl:text>
						<xsl:for-each select='$rootmenu/a'>
							<xsl:if test='position() &gt; 1'> | </xsl:if>
							<xsl:copy-of select='.'/>
						</xsl:for-each>
					</xsl:if>
				</span><br />
				<!-- The title. -->

				<xsl:comment> google_ad_section_start </xsl:comment>
				<span class='headslug'><xsl:value-of select='@title'/></span><br />
				<xsl:comment> google_ad_section_end </xsl:comment>

				<!-- Below the title: the page path. -->
				<span class='path'>
					<xsl:copy-of select='$pagepath/*|$pagepath/text()'/>
				</span>
			</td>
		</tr>
		<tr>
			<td id='menu' valign='top'>

				<xsl:comment> google_ad_section_start(weight=ignore) </xsl:comment>

				<div class='sidebar'>

					<!-- Find a left-side sidebar menu for this page. -->

					<xsl:choose>
						<!-- If PHP isn't an option, use the file. -->
						<xsl:when test='$nophp'>
							<xsl:choose>
								<xsl:when test='pagemenu/@file'>
									<xsl:apply-templates select='document(pagemenu/@file)/*' />
								</xsl:when>
								<xsl:otherwise>
									<xsl:apply-templates select='document("sidebar.hx")/*' />
								</xsl:otherwise>
							</xsl:choose>
						</xsl:when>
						<!-- If there's a PHP include option, use it. -->
						<xsl:when test='pagemenu/@phpinc'>
							<xsl:call-template name='php'>
								<xsl:with-param name='code'>
									include('<xsl:value-of select='pagemenu/@phpinc'/>');
								</xsl:with-param>
							</xsl:call-template>
						</xsl:when>
						<xsl:otherwise>
							<xsl:call-template name='php'>
								<xsl:with-param name='code'>
									include('/home/nedbatc/public_html/sidebar.inc');
								</xsl:with-param>
							</xsl:call-template>
						</xsl:otherwise>
					</xsl:choose>

				</div>

				<xsl:comment> google_ad_section_end </xsl:comment>

			</td>

			<td id='content' valign='top'>

				<div class='body'>
					<!-- An empty div for main.js to write a search intro into. -->
					<div id='searchwelcome'/>

					<xsl:if test='//history/what/text()'>
						<p class='dateline'>
							Created
							<xsl:call-template name='long_date'>
								<xsl:with-param name='date' select='//history/what[1]/@when' />
							</xsl:call-template>
							<xsl:if test='//history/what[2]'>
								<xsl:text>, last updated </xsl:text>
								<xsl:call-template name='long_date'>
									<xsl:with-param name='date' select='//history/what[last()]/@when' />
								</xsl:call-template>
							</xsl:if>
						</p>
					</xsl:if>

					<xsl:comment> google_ad_section_start </xsl:comment>

					<xsl:apply-templates />

					<xsl:comment> google_ad_section_end </xsl:comment>

				</div>

			</td>
		</tr>
		<tr>
			<td/>
			<td id='copyright'>
				<xsl:call-template name='copyright'/>
			</td>
		</tr>

		</table>

	</body>

</xsl:template>

<!-- Compute the proper copyright notice. -->
<xsl:template name='copyright'>

	<xsl:variable name='min_when'>
		<xsl:call-template name='string_min'>
			<xsl:with-param name='nodes' select='//history/what/@when' />
			<xsl:with-param name='default' select='"20020307T120000"' />
		</xsl:call-template>
	</xsl:variable>
	<xsl:variable name='max_when'>
		<xsl:call-template name='string_max'>
			<xsl:with-param name='nodes' select='//history/what/@when' />
			<xsl:with-param name='default' select='xuff:now()' />
		</xsl:call-template>
	</xsl:variable>

	<xsl:variable name='min_year'>
		<xsl:value-of select='substring($min_when,1,4)' />
	</xsl:variable>
	<xsl:variable name='max_year'>
		<xsl:value-of select='substring($max_when,1,4)' />
	</xsl:variable>

	<p class='copyright'>
		<a href='site/legal.html' rel='copyright'>
			<xsl:text>&#xa9; Copyright </xsl:text>
			<xsl:value-of select='$min_year' />
			<xsl:if test='$min_year != $max_year'>
				<xsl:text>-</xsl:text>
				<xsl:value-of select='$max_year' />
			</xsl:if>
			<xsl:text>, Ned Batchelder</xsl:text>
		</a>
	</p>

</xsl:template>

</xsl:stylesheet>
